/**
 * Main definitions and such pulling together the pieces we need for
 *   the project 4 raycaster
 *
 * @author: Konstantin Shkurko (kshkurko@cs.utah.edu)
 * @date:	10.06.11
 */
#ifndef PROJECT3_H
#define PROJECT3_H

// Only include stdio for printf on the non-trax version
#if TRAX==0
#include <stdio.h>
#include <stdlib.h>
#endif

#include "trax.hpp"
#include "Constants.h"
#include "Color.h"
#include "gpBox.h"
#include "gpTriangle.h"
#include "gpSphere.h"
#include "myCamera.h"
#include "IntersectionRecord.h"
#include "RayPrimitive.h"
#include "MatPrimitive.h"
#include "ltPoint.h"


// main render function to test pthreads...
inline void * renderMe( void * );



/**
 * Helper function that helps us to store a pixel into the frame buffer
 *
 * @param fb	- framebuffer where we're saving everything into
 * @param pixel - pixel index where to store everything into (aka offset from start)
 * @param clr	- color to store
 */
inline void StorePixelIntoFB( const int &fb, const int &pixel, const Color &clr );


/**
 * Helper function that throws a ray through our scene and traverses
 *   the BVH, while keeping track of the intersection record
 * NOTE: this works only for the TRaX triangle-only scenes
 *
 * @param ray      - ray to cast through the BVH (will be updated with far t-value)
 * @param intRec   - record holding intersection information
 * @param sAddrBVH - start address of the BVH
 * @return flag if anything was intersected with ray
 */
inline bool intersectDetailedWithBVH( RayPrimitive &ray, IntersectionRecord &intRec, const int &sAddrBVH );


/**
 * Quick intersection of a ray with the BVH scene. This is intended for 
 *   shadow rays only. Returns whether a ray has hit anything
 * NOTE: this works only for the TRaX triangle-only scenes
 *
 * @param ray - ray to cast through the BVH (will be updated with far t-value)
 * @param sAddrBVH - start address of the BVH
 * @return flag whether ray hits anything
 */
inline bool intersectQuickWithBVH( RayPrimitive &ray, const int &sAddrBVH );


#if defined(DO_MULTIOBJECT) && defined(DO_REFLECTIONS)
inline void ComputeRecursiveRays( Color &cOut, RayPrimitive &ruPRay, RayPrimitive &ruSRay, IntersectionRecord &reuseIntRec,
								 const unsigned int &level, const int &numSph, gpSphere ** spheres,
								 const int &numTri, gpTriangle ** tris, const Color &ambLt,
								 const int &numLt, ltPoint ** ptLts, const int &sAddrBVH, const int &sAddrMat,
								 const unsigned int &maxLvl=7, const float &kAmb=0.3f, const float &kDiff=0.7f );

inline bool computeTransmittedDirection( Vector &tDir, const Vector &inDir, const Vector &norm, 
											const float &dDotN, const float &etaFrac );//n, const float &nt );
#endif


#endif // PROJECT4_H
